<?php

namespace Drupal\barclaycard_epdq\Plugin\Commerce\PaymentGateway;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\barclaycard_epdq\BarclaycardEdpqTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;

/**
 * Provides the Barclaycard ePDQ offsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "barclaycard_epdq",
 *   label = @Translation("Barclaycard ePDQ"),
 *   display_label = @Translation("Barclaycard ePDQ"),
 *   modes = {
 *     "n/a" = @Translation("N/A"),
 *   },
 *   forms = {
 *     "offsite-payment" = "Drupal\barclaycard_epdq\PluginForm\BarclaycardEpdqPaymentForm",
 *   },
 *   credit_card_types = {
 *     "mastercard", "visa", "maestro",
 *   },
 * )
 */
class BarclaycardEpdq extends OffsitePaymentGatewayBase {

  use BarclaycardEdpqTrait;

  /**
   * Constructs a new PaymentGatewayBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = [
      'vat_percentage'     => 0,
      'redirect_url'       => '',
      'pspid'              => '',
      'sha_in_passphrase'  => '',
      'sha_out_passphrase' => '',
      'accept_url'         => '',
      'decline_url'        => '',
      'exception_url'      => '',
      'cancel_url'         => '',
      'back_url'           => '',
      'home_url'           => '',
    ] + parent::defaultConfiguration();

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['vat_percentage'] = [
      '#type'          => 'number',
      '#title'         => $this->t('VAT percentage'),
      '#description'   => $this->t('The VAT percentage to add to the order amount (defaults to 0).'),
      '#default_value' => $this->configuration['vat_percentage'],
      '#required'      => FALSE,
    ];

    $form['redirect_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Redirect URL'),
      '#description'   => $this->t('This is the payment page the user will be redirected to.'),
      '#default_value' => $this->configuration['redirect_url'],
      '#required'      => TRUE,
    ];

    $form['pspid'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('PSPID'),
      '#description'   => $this->t('Your Barclaycard ePDQ account PSPID'),
      '#default_value' => $this->configuration['pspid'],
      '#required'      => TRUE,
    ];

    $form['sha_in_passphrase'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('SHA in passphrase'),
      '#description'   => $this->t('The SHA in passphrase for your Barclaycard ePDQ account.'),
      '#default_value' => $this->configuration['sha_in_passphrase'],
      '#required'      => TRUE,
    ];

    $form['sha_out_passphrase'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('SHA out passphrase'),
      '#description'   => $this->t('The SHA out passphrase for your Barclaycard ePDQ account.'),
      '#default_value' => $this->configuration['sha_out_passphrase'],
      '#required'      => TRUE,
    ];

    $form['accept_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Accept URL'),
      '#description'   => $this->t('The URL that users are redirected to after paying for their order.'),
      '#default_value' => $this->configuration['accept_url'],
      '#required'      => FALSE,
    ];

    $form['decline_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Decline URL'),
      '#description'   => $this->t('The URL users are redirected to if their payment has been declined the maximum allowed times.'),
      '#default_value' => $this->configuration['decline_url'],
      '#required'      => FALSE,
    ];

    $form['exception_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Exception URL'),
      '#description'   => $this->t('The URL users are redirected to if an error occurs when they are paying.'),
      '#default_value' => $this->configuration['exception_url'],
      '#required'      => FALSE,
    ];

    $form['cancel_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Cancel URL'),
      '#description'   => $this->t('The URL users are redirected to if they click cancel on the payment page.'),
      '#default_value' => $this->configuration['cancel_url'],
      '#required'      => FALSE,
    ];

    $form['back_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Back URL'),
      '#description'   => $this->t('The back URL.'),
      '#default_value' => $this->configuration['back_url'],
      '#required'      => FALSE,
    ];

    $form['home_url'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Home URL'),
      '#description'   => $this->t('The home URL.'),
      '#default_value' => $this->configuration['home_url'],
      '#required'      => FALSE,
    ];

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValue($form['#parents']);
    $this->configuration['vat_percentage'] = $values['vat_percentage'];
    $this->configuration['redirect_url'] = $values['redirect_url'];
    $this->configuration['pspid'] = $values['pspid'];
    $this->configuration['sha_in_passphrase'] = $values['sha_in_passphrase'];
    $this->configuration['sha_out_passphrase'] = $values['sha_out_passphrase'];
    $this->configuration['accept_url'] = $values['accept_url'];
    $this->configuration['decline_url'] = $values['decline_url'];
    $this->configuration['exception_url'] = $values['exception_url'];
    $this->configuration['cancel_url'] = $values['cancel_url'];
    $this->configuration['back_url'] = $values['back_url'];
    $this->configuration['home_url'] = $values['home_url'];
  }

}
